/****************************************************************************
** $Id: qsinterpreter.cpp  beta2   edited Jan 29 14:42 $
**
** Implementation of the QSInterpreter class.
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsinterpreter.h"
#include "qsinterfacefactory_p.h"
#include "../kernel/quickdebugger.h"
#include "../kernel/quickinterpreter.h"
#include "../kernel/quickobjects.h"
#include "../engine/qsenv.h"
#include "../engine/qsfuncref.h"

#include <qapplication.h>
#include <qmessagebox.h>
#include <qptrlist.h>

// Designer classes
#include <formfile.h>
#include <formwindow.h>
#include <mainwindow.h>
#include <project.h>


class QSInterpreterPrivate
{
public:
};

/*!
  \class QSInterpreter qsinterpreter.h

  \brief The QSInterpreter class provides the public API for Qt Script
  for Applications script engine.

  This class (implemented in \c libqsa) provides the functionality
  required to make Qt/C++ applications scriptable with Qt Script.

  For convience reasons, a single instance of the QSInterpreter class
  exists in an application; it is available as
  QSInterpreter::defaultInterpreter().

  Use addObject() to make application objects available to the
  interpreter for use by scripts. Such objects can be removed from the
  interpreter's view with removeObject().

  A global function can be executed using call(). Any string
  containing valid Qt Script code can be executed using evaluate().
  These two functions do not require a project to be open, but most
  other functions only work if there is a current project. Even if you
  don't use the Qt Scripter, it is suggested to use a scripting
  project, only for very simple cases, where nothing more than
  evaluating a simple piece of code is needed, it makes sense to work
  without a project.

  If an error occurs, e.g. during the execution of a script, the
  error() signal is emitted. The error behavior depends on the
  errorMode() which is set with setErrorMode().

  The list of functions available in a particular class is available
  from functionsOf(), and the list of global functions is returned by
  globalFunctions(). To introspect variables and classes in a context,
  use classesOf() and variablesOf().

  \omit
  The Qt Script package paths can be modified and retrieved using
  addPackagePath(), removePackagePath() and packagePaths(). These
  paths are used by Qt Script to locate packages which can be imported
  using the \c import statement.
  \endomit

  See the \link qsa.book Manual\endlink for more explanation and
  examples.

*/

/*! \fn bool QSInterpreter::isRunning() const
  Return TRUE if the interpreter is currently evaluating code, returns
  FALSE otherwise.
*/


static QuickInterpreter *quickInterpreterPtr = 0;


/*!
  Constructs a QSInterpreter.

  Currently, only one instance of the class QSInterpreter may exist at
  any one time. You can access this instance with
  QSInterpreter::defaultInterpreter().

  \a project specifies the QSProject, which this interpreter will work
  on.
*/

QSInterpreter::QSInterpreter( QSProject *project )
    :
    em( AskForDebug ),
    d( new QSInterpreterPrivate ),
    proj( project ? project : new QSProject() ),
    signalHandlers( new QValueList<QSSignalHandler>() ),
    running( FALSE ),
    hadError( FALSE ),
    blockErrors( FALSE ),
    needRerun( FALSE )
{
    if ( !quickInterpreterPtr )
	quickInterpreterPtr = new QuickInterpreter( TRUE );
    quickinter = quickInterpreterPtr;
    proj->setInterpreter( this );
    ifaceFactory = new QSInterfaceFactory( this );
    connect( quickinter, SIGNAL( runtimeError() ), this, SLOT( runtimeError() ) );
}

/*! Destructor
  \internal
*/

QSInterpreter::~QSInterpreter()
{
    delete d;
    if ( quickinter == quickInterpreterPtr )
	quickInterpreterPtr = 0;
    delete quickinter;
    delete signalHandlers;
    delete ifaceFactory;
}

/*!
  Returns the application-wide default interpreter.

  (This function will automatically create the interpreter if it
  doesn't already exist.)
 */

QSInterpreter* QSInterpreter::defaultInterpreter()
{
    // ### Not good!!
    static QSInterpreter *inter = new QSInterpreter();
    return inter;
}

/*! \property QSInterpreter::errorMode
  \brief what should happen when an error occurs

*/

/*! \enum QSInterpreter::ErrorMode

  The ErrorMode describes what should happen when an error occurs
  while parsing or executing script code.

  \value Notify Notifies the user that an error occurred via a message
  box.

  \value AskForDebug Notifies the user that an error occurred via a
  message box and offers them the option of opening Qt Scripter to
  debug the problem. Qt Scripter can be only opened if a scripting
  project is open.<sup>*</sup>

  \value Debug Opens Qt Scripter to debug the problem. Qt Scripter
  can be only opened if a scripting project is open.<sup>*</sup>

  \value Nothing Nothing happens.

  <sup>*</sup> If there is no project open, Qt Scripter can and will
  not be invoked, so you will need to handle the error() signal
  yourself or only get a notification message box, if the errorMode()
  is something else than Nothing.
*/

void QSInterpreter::setErrorMode( ErrorMode m )
{
    em = m;
}

QSInterpreter::ErrorMode QSInterpreter::errorMode() const
{
    return em;
}

/*! \fn void QSInterpreter::error( const QString &message,
  const QString &filename, int lineNumber )

  This signal is emitted if an error occurs when running or parsing a
  script. \a message contains the error message from the interpreter,
  \a filename the filename (if known) where the error occured, and \a
  lineNumber the line number at which the error occured..
*/

/*! \fn void QSInterpreter::error( const QString &message,
  QObject *context, const QString &filename, int lineNumber )

  This signal is emitted if an error occurs when running or parsing a
  script. \a message contains the error message from the interpreter,
  \a context a pointer to the QObject context in which the error
  occured or 0, if the context is the global context, \a filename the
  filename (if known) where the error occured, and \a lineNumber the
  line number at which the error occured..
*/

/*!
  Executes the string of Qt Script in \a code and returns any value
  produced by that \a code.

  This function executes the code passed in as \a code. The code can
  use and reference code (functions, classes, variables, etc.) which
  have been passed to this function previously or which are defined in
  the current project. Also application objects, which have been added
  via addObject() can be accessed.

  If \a context is 0 (default), the code is executed as global
  code. If a \a context is given, the code is executed in the context
  of that object.

  The functions openIDE() and closeIDE() clear the
  interpreter and re-evaluate the current project, meaning the code
  which has been passed previously into evaluate gets lost when
  calling one of these functions.

*/

QVariant QSInterpreter::evaluate( const QString &code, QObject *context )
{
    if ( running ) {
	qWarning( "QSInterpreter::evaluate: cannot recursively call"
		  " evaluate(), call() or checkSyntax()" );
	return QVariant();
    }
    running = TRUE;
    proj->checkProject();
    int id = 0;
    QVariant v = interpreter()->execute( context, code, id );
    proj->stopProject();
    running = FALSE;
    return v;
}

/*!
  Calls the function \a function with the given \a arguments. The
  arguments are converted into Qt Script datatypes first.

  Functions which were passed to evaluate() in previous calls or which
  are defined in the current project, can be called from this
  function.

  If \a context is 0 (default), the function is called in the global
  scope. If a \a context is given, the function is called in the scope
  of that object.

  The functions openIDE() and closeIDE() clear the
  interpreter and re-evaluate the current project, meaning the code
  which has been passed previously into evaluate() gets lost when
  calling one of these functions.

*/

QVariant QSInterpreter::call( const QString &function,
				    const QValueList<QVariant> &arguments,
				    QObject *context )
{
     if ( running ) {
	qWarning( "QSInterpreter::call: cannot recursively call"
		  " call(), evaluate() or checkSyntax()" );
	return QVariant();
    }
    running = TRUE;
    proj->checkProject();
    QVariant v = interpreter()->call( context, function, arguments );
    proj->stopProject();
    running = FALSE;
    return v;
}

/*!  Checks if the script code \a code is free of syntax errors or
  not. Returns TRUE if the code is free of syntax errors, otherwise
  FALSE.
*/

bool QSInterpreter::checkSyntax( const QString &code )
{
    if ( running ) {
	// ### check whether this limitation still exists
	qWarning( "QSInterpreter::checkSyntax: cannot recursively call"
		  " call(), evaluate() or checkSyntax()" );
	return FALSE;
    }
    running = TRUE;
    proj->checkProject();
    bool ok = interpreter()->checkSyntax( code );
    proj->stopProject();
    running = FALSE;
    return ok;
}

/*! Returns the current execution context of the interpreter. This is
  either a QObject pointer or 0.
*/

QObject *QSInterpreter::currentContext() const
{
    QSObject o = interpreter()->env()->currentScope();
    if ( o.isA( interpreter()->wrapperClass() ) )
	return interpreter()->wrapperClass()->shared( &o )->objects[0];
    return 0;
}

/*! Returns the current project of the interpreter or 0 if there is
  no project.
*/
QSProject *QSInterpreter::project() const
{
    return proj;
}

/*! \internal */
QuickInterpreter *QSInterpreter::interpreter() const
{
    return quickinter;
}

/* \internal */
QSInterfaceFactory *QSInterpreter::interfaceFactory() const
{
    return ifaceFactory;
}


/*!

  Makes the QObject \a o available to the scripting engine. All child
  objects of \a o are made available too.

  If no object in the parent hierarchy of \a o has been added via
  addObject() yet, \a o will be made available as a toplevel object to
  the programmer and will be accessible via \c Application.object_name
  (where \c object_name is the value of \e o's QObject::name()
  property).

  If an object in the parent hierarchy of \a o has been made available
  via addObject() already, \a o will not be made available as a
  toplevel object. It is accessible then through \c
  parent1.parent2.object_name in the scripting language, given that \c
  parent1 has been made available via addObject() previously. The
  reason to make an object available like this, even though it is not
  made available as a toplevel object, is so that code can be added in
  in the context of that object. See setObjectSource().

  \warning Every object passed to this function must have a name.
*/

void QSInterpreter::addObject( QObject *o )
{
    if ( !o )
	return;
#if defined(QT_CHECK_RANGE)
    if ( strcmp( o->name(), "unnamed" ) == 0 )
	qWarning( "QSInterpreter::addObject: unnamed object" );
#endif
    connect( o, SIGNAL( destroyed( QObject * ) ), this, SLOT( objectDestroyed( QObject * ) ) );
    interpreter()->addTopLevelObject( o );
    appObjs.append( o );
    // ### ! Designer classes
    if ( proj->project )
	proj->project->addObject( o );
}

/*! Removes the QObject \a o from the list of available objects for
  scripting.

  \sa clearObjects()
*/

void QSInterpreter::removeObject( QObject *o )
{
    // ### ! Designer classes
    if ( appObjs.removeRef( o ) && proj->project )
	proj->project->removeObject( o );
    QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
    while ( it != signalHandlers->end() ) {
	QValueList<QSSignalHandler>::Iterator it2 = it;
	QSSignalHandler sigh = *it2;
	++it;
	if ( sigh.sender == o || sigh.receiver == o )
	    signalHandlers->remove( it2 );
    }
}

/*!
  Returns the list of available objects for scripting in this interpreter.
*/
const QObjectList* QSInterpreter::objects() const
{
    return &appObjs;
}

/*! Removes all application objects from the list of available
  objects. Note that this deosn't delete the application objects!

  \sa removeObject()
*/

void QSInterpreter::clearObjects()
{
    while( appObjs.count() )
	removeObject( appObjs.last() );
}

/*! Clears the interpreter and closes the currently opened Qt Script
  for Applications project, if one is open.

  This function does not clear the application objects. Use
  clearObjects() to do that.
*/

void QSInterpreter::clear()
{
    proj->closeProject();
    interpreter()->emitStopProject();
    interpreter()->clear();
    running = FALSE;

}

/*! Adds the Qt Script function \a qtscriptFunction in the context of
  \a receiver as signal handler for the C++ signal \a signal of the
  object \a sender.

  Example:
  \code
  QSInterpreter::defaultInterpreter()->addSignalHandler( Form1.okButton, SIGNAL( clicked() ), document, "startCalculation" );
  \endcode

  \sa removeSignalHandler()
*/

void QSInterpreter::addSignalHandler( QObject *sender, const char *signal,
				      QObject *receiver,
				      const char *qtscriptFunction )
{
    proj->checkProject();
    QuickInterpreter *ip = interpreter();
    QString func( qtscriptFunction );
    QSObject senderObj = ip->wrap( sender );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->setEventHandler( ip, signal + 1, receiver, func.left( func.find( '(' ) ) );

    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = receiver;
    sigh.signal = signal + 1;
    sigh.function = func.left( func.find( '(' ) );
    (*signalHandlers) << sigh;

    connect( sender, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
    connect( receiver, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
}

/*! Adds the Qt Script function \a qtscriptFunction (fully qualified)
  as signal handler for the C++ signal \a signal of the object \a
  sender.

  Example:
  \code
  QSInterpreter::defaultInterpreter()->addSignalHandler( Form1.okButton, SIGNAL( clicked() ), "classA.obj.calculate" );
  \endcode

  \sa removeSignalHandler()
*/

void QSInterpreter::addSignalHandler( QObject *sender, const char *signal,
				      const char *qtscriptFunction )
{
    proj->checkProject();

    QuickInterpreter *ip = interpreter();
    QString func( qtscriptFunction );
    func.left( func.find( '(' ) );
    QSObject senderObj = ip->wrap( sender );

    QSObject obj = ip->object( func );
    QSObject base = QSFuncRefClass::refBase( obj );
    QSMember member = QSFuncRefClass::refMember( obj );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->setEventHandler( ip, signal + 1, 0, member.name(), base );

    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = 0;
    sigh.signal = signal + 1;
    sigh.function = func;
    (*signalHandlers) << sigh;

    connect( sender, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
}

/*! Removes the connection between the signal \a signal of the object
  \a sender and the the signal handler \a qtscriptFunction in the
  context \a receiver.

  \sa addSignalHandler()
*/

void QSInterpreter::removeSignalHandler( QObject *sender, const char *signal,
					 QObject *receiver,
					 const char *qtscriptFunction )
{
    proj->checkProject();
    QString func( qtscriptFunction );
    QuickInterpreter *ip = interpreter();
    QSObject senderObj = ip->wrap( sender );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->removeEventHandler( signal + 1, receiver, func.left( func.find( '(' ) ) );
    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = receiver;
    sigh.signal = signal + 1;
    sigh.function = func.left( func.find( '(' ) );
    for ( QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
	  it != signalHandlers->end(); ++it ) {
	if ( sigh == *it ) {
	    signalHandlers->remove( it );
	    break;
	}
    }
}

/*! Removes the connection between the signal \a signal of the object
  \a sender and the the fully qualified signal handler \a
  qtscriptFunction.

  \sa addSignalHandler()
*/

void QSInterpreter::removeSignalHandler( QObject *sender, const char *signal,
					       const char *qtscriptFunction )
{
    proj->checkProject();

    QuickInterpreter *ip = interpreter();
    QString func( qtscriptFunction );
    func.left( func.find( '(' ) );
    QSObject senderObj = ip->wrap( sender );

    QSObject obj = ip->object( func );
    QSObject base = QSFuncRefClass::refBase( obj );
    QSMember member = QSFuncRefClass::refMember( obj );
    QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
    sh->removeEventHandler( signal + 1, 0, member.name(), base );

    QSSignalHandler sigh;
    sigh.sender = sender;
    sigh.receiver = 0;
    sigh.signal = signal + 1;
    sigh.function = func;
    for ( QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
	  it != signalHandlers->end(); ++it ) {
	if ( sigh == *it ) {
	    signalHandlers->remove( it );
	    break;
	}
    }
}

/*!
  If a project is open, then all the names of all the global functions
  in that project are returned. If no project is open, then all the
  names of all the global functions created with evaluate() are
  returned.

 If \a includeSignature is FALSE (the default), no signature is
 appended to the function name. If \a includeSignature is TRUE then
 each name returned will be of the form:
 \code
 functionName( typeOfArg1, typeOfArg2, ... )
 \endcode

 \sa functionsOf(), classes(), classesOf(), variablesOf()
*/

QStringList QSInterpreter::globalFunctions( bool includeSignature ) const
{
    return functionsOf( QString::null, includeSignature );
}

/*!  Returns all script functions in the context \a context (this can
  be e.g. a class or form). If \a context is empty, the functions of
  the global context (global functions) are returned.

  \a context can be fully qualified.

  If \a includeSignature is FALSE (the default), no signature is
  appended to the function name. If \a includeSignature is TRUE then
  each name returned will be of the form:
  \code
  functionName( typeOfArg1, typeOfArg2, ... )
  \endcode

  \sa globalFunctions(), classes(), classesOf(), variablesOf()
*/

QStringList QSInterpreter::functionsOf( const QString &context,
					bool includeSignature ) const
{
    proj->checkAndStopProject();
    QSObject obj = interpreter()->object( context );
    return interpreter()->functionsOf( obj, includeSignature );
}

/*! Returns all script functions which have been defined in the
  context \a context.

  If \a includeSignature is FALSE (the default), no signature is
  appended to the function name. If \a includeSignature is TRUE then
  each name returned will be of the form:
  \code
  functionName( typeOfArg1, typeOfArg2, ... )
  \endcode

  \sa globalFunctions(), classes(), classesOf(), variablesOf()
*/

QStringList QSInterpreter::functionsOf( QObject *context,
					bool includeSignature ) const
{
    proj->checkAndStopProject();
    QSObject obj = interpreter()->wrap( context );
    if ( obj.isUndefined() )
	return QStringList();
    return interpreter()->functionsOf( obj, includeSignature );
}

/*!
  Returns all currently declared classes.

  \sa functionsOf(), classesOf(), variablesOf()
*/

QStringList QSInterpreter::classes() const
{
    proj->checkAndStopProject();
    return interpreter()->classes();
}

/*! Returns all currently declared classes in the fully qualified
  context \a context

  \sa functionsOf(), classes(), globalClasses(), variablesOf()
*/

QStringList QSInterpreter::classesOf( const QString &context ) const
{
    proj->checkAndStopProject();
    QSObject obj = interpreter()->object( context );
    return interpreter()->classesOf( obj );
}

/*! Returns all currently declared classes in the context \a context

  \sa functionsOf(), classes(), globalClasses(), variablesOf()
*/

QStringList QSInterpreter::classesOf( QObject *context ) const
{
    proj->checkAndStopProject();
    QSObject obj = interpreter()->wrap( context );
    if ( obj.isUndefined() )
	return QStringList();
    return interpreter()->classesOf( obj );
}

/*! Returns all currently declared classes in the global context

  \sa functionsOf(), classes(), classesOf(), variablesOf()
*/

QStringList QSInterpreter::globalClasses() const
{
    return classesOf( QString::null );
}

/*! Returns all currently declared variables in the fully qualified
  context \a context

  \sa functionsOf(), classes(), classesOf(), globalFunctions()
*/

QStringList QSInterpreter::variablesOf( const QString &context ) const
{
    proj->checkAndStopProject();
    QSObject obj = interpreter()->object( context );
    return interpreter()->variablesOf( obj );
}

/*! Returns all currently declared variables in the contenxt \a context

  \sa functionsOf(), classes(), classesOf(), globalFunctions()
*/

QStringList QSInterpreter::variablesOf( QObject *context ) const
{
    proj->checkAndStopProject();
    QSObject obj = interpreter()->wrap( context );
    if ( obj.isUndefined() )
	return QStringList();
    return interpreter()->variablesOf( obj );
}

/*! Returns all currently declared variables in the global context

  \sa functionsOf(), classes(), classesOf(), variablesOf()
*/

QStringList QSInterpreter::globalVariables() const
{
    return variablesOf( QString::null );
}

/*! \internal */
void QSInterpreter::readdObjects()
{
    QuickInterpreter *ip = interpreter();
    for ( QObject *o = appObjs.first(); o; o = appObjs.next() )
	ip->addTopLevelObject( o );
    QPtrListIterator<FormFile> it = project()->project->formFiles();
    while ( it.current() ) {
	if ( !it.current()->isFake() && it.current()->formWindow() )
	    ip->addTopLevelObject( it.current()->formWindow()->mainContainer() );
	++it;
    }
}

/*! \internal */
void QSInterpreter::readdEventHandlers()
{
    QuickInterpreter *ip = interpreter();
    for ( QValueList<QSSignalHandler>::Iterator it = signalHandlers->begin();
	  it != signalHandlers->end(); ++it ) {
	QSSignalHandler sigh = *it;
	QSObject senderObj = ip->wrap( sigh.sender );
	QSWrapperShared *sh = ip->wrapperClass()->shared( &senderObj );
	if ( sigh.receiver ) {
	    sh->setEventHandler( ip, sigh.signal, sigh.receiver, sigh.function );
	} else {
	    QSObject obj = ip->object( sigh.function );
	    QSObject base = QSFuncRefClass::refBase( obj );
	    QSMember member = QSFuncRefClass::refMember( obj );
	    sh->setEventHandler( ip, sigh.signal, 0, member.name(), base );
	}
    }
}

void QSInterpreter::runtimeError()
{

    if ( blockErrors || proj->project )
	return;
    int l = interpreter()->debuggerEngine()->lineNumber();
    if ( l < 0 )
	l = interpreter()->errorLines().first();
    runtimeError( interpreter()->errorMessages().first(),
		  "<unknown>",
		  l );
}

void QSInterpreter::runtimeError( const QString &message,
			      const QString &filename,
			      int lineNumber )
{
    if ( blockErrors )
	return;

    hadError = TRUE;
    emit error( message, filename, lineNumber );
    QObject *ctx = interpreter()->objectOfSourceId( interpreter()->debuggerEngine()->sourceId() );
    emit error( message, ctx, filename, lineNumber );

    if ( !proj->isOpened() && qApp->type() != QApplication::Tty ) {
	if ( em != Nothing ) {
	    QMessageBox::critical( qApp->mainWidget(), "Error", "The following error occurred in "
				   "line <b>" + QString::number( lineNumber ) +
				   "</b> of file <b>" + filename +
				   "</b> while executing "
				   "the script:<pre><font color=red>" +
				   message + "</font></pre>" );
	}
	return;
    }

    interpreter()->emitStopProject();
    proj->running = FALSE;

    if ( qApp->type() == QApplication::Tty )
	return;

    bool debug = FALSE;
    if ( !proj->designer || !proj->designer->isVisible() ) {
	switch ( em ) {
	case Notify:
	    QMessageBox::critical( qApp->mainWidget(), "Error", "The following error occurred in "
				   "line <b>" + QString::number( lineNumber ) +
				   "</b> of file <b>" + filename +
				   "</b> while executing "
				   "the script:<pre><font color=red>" +
				   message + "</font></pre>" );
	    break;
	case AskForDebug:
	    debug = QMessageBox::critical( qApp->mainWidget(), "Error", "The following error occurred in "
					   "line <b>" + QString::number( lineNumber ) +
					   "</b> of file <b>" + filename +
					   "</b> while executing "
					   "the script:<pre><font color=red>" +
					   message + "</font></pre>"
					   "Do you want to debug the script?", "&Yes", "&No",
					   "", 0, 1 ) == 0;
	    break;
	case Debug:
	    debug = TRUE;
	    break;
	case Nothing:
	    break;
	}
    } else {
	debug = TRUE;
    }

    if ( debug ) {
	if( QSProject::isIDEOpen() && QSProject::ideInterpreter() != this ) {
	    QMessageBox::critical( qApp->mainWidget(), "Unable to debug script error",
				   "The Qt Scritper IDE is already opened with another project. You have to "
				   "close the IDE and rerun the script to debug it.",
				   QMessageBox::Ok, QMessageBox::NoButton );
	    return;
	}
	if ( !proj->designer )
	    proj->setupDesigner();
	proj->designer->show();
	proj->designer->raise();
    }
}


void QSInterpreter::runFinished()
{
    proj->running = FALSE;
    running =  FALSE;
}

void QSInterpreter::objectDestroyed( QObject *obj )
{
    removeObject( obj );
}



