/****************************************************************************
** $Id: qsobjectfactory.cpp  beta2   edited Jan 27 14:30 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsobjectfactory.h"
#include "qsinterfacefactory_p.h"
#include "qsinterpreter.h"
#include "quickinterpreter.h"
#include "qsenv.h"

/*!
  \class QSArgument qsobjectfactory.h

  \brief The QSArgument class provides storage for various datatypes
  used in QSObjectFactory::create() calls.

  When the user writes Qt Script code like this:
  \code
    var x = new AnotherCppObject( arg1, arg2 ); // Qt Script
  \endcode
  the arguments are passed to the QSObjectFactory::create() call as a
  QValueList of QSArguments. A QSArgument can hold a QVariant, a
  QObject pointer or a void pointer. The type held by a QSArgument is
  returned by type() and the value by variant(), qobject() or ptr() as
  appropriate.

*/

/*! \enum QSArgument::Type

   This enum describes which datatype is held by this QSArgument.

   \value Invalid Invalid datatype
   \value Variant Holds a QVariant
   \value QObjectPtr Holds a QObject pointer
   \value VoidPointer Holds an untyped pointer (void*)
*/

/*! Creates an invalid QSArgument */

QSArgument::QSArgument()
{
    qobj = 0;
    pt = 0;
    typ = Invalid;
}

/*!
  Creates a QSArgument which holds the variant \a v as its value and
  is of type Variant.
*/

QSArgument::QSArgument( const QVariant &v )
{
    var = v;
    qobj = 0;
    pt = 0;
    typ = Variant;
}

/*!
  Creates a QSArgument which holds the QObject pointer \a o as its value
  and is of type QObjectPtr.
*/

QSArgument::QSArgument( QObject *o )
{
    qobj = o;
    pt = 0;
    typ = QObjectPtr;
}

/*!
  Creates a QSArgument which holds the pointer \a p as its value and
  is of type VoidPointer.
*/

QSArgument::QSArgument( void *p )
{
    pt = p;
    qobj = 0;
    typ = VoidPointer;
}

/*!
    If this QSArgument's type() is Variant, returns the variant value;
    otherwise returns an invalid variant.
*/

QVariant QSArgument::variant() const
{
    if ( typ == Variant )
	return var;
    return QVariant();
}

/*!
    If this QSArgument's type() is QObjectPtr, returns the object
    pointer; otherwise returns 0.
*/

QObject *QSArgument::qobject() const
{
    if ( typ == QObjectPtr )
	return qobj;
    return 0;
}

/*!
    If this QSArgument's type() is VoidPointer, returns the void pointer;
    otherwise returns 0.
*/
void *QSArgument::ptr() const
{
    if ( typ == VoidPointer )
	return pt;
    return 0;
}

/*!
  Returns the type of this QSArgument. This can be \c Invalid, \c
  Variant, \c QObjectPtr or \c VoidPointer

  \sa typeName()
*/

QSArgument::Type QSArgument::type() const
{
    return typ;
}

/*! Returns the type name of this argument. */

QString QSArgument::typeName() const
{
    switch ( typ ) {
    case Invalid:
	return "Invalid";
    case Variant:
	return variant().typeName();
    case QObjectPtr:
	return qobject()->className();
    case VoidPointer:
	return "Pointer";
    }
    return "Unknown";
}

/*!
  \class QSObjectFactory qsobjectfactory.h

  \brief The QSObjectFactory class provides a mechanism for Qt Script
  programs to create C++ QObjects.

  If we want script programmers to create their own C++ QObjects we
  can either provide a QObject subclass that has a slot which is a
  factory function that returns QObjects, or we can subclass
  QSObjectFactory and reimplement the create() and classes()
  functions.

  The classes() function is called by the scripting engine to see
  which classes can be construced by the script programmer. And the
  create() function is called by the scripting engine to create the
  instance, i.e. when the user writes something like this:
  \code
    var x = new SomeCppObject( arg1, arg2 ); // Qt Script
  \endcode

  A single QSObjectFactory subclass may be used to provide any number of
  creatable QObject classes. To make these classes known to the
  scripting engine, create an instance of the subclass with a pointer
  to the interpreter as an argument.

*/

/*! \fn QObject *QSObjectFactory::create( const QString &className, const QValueList<QSArgument> &arguments, QObject *context )

  This pure virtual function should be reimplemented in your
  QSObjectFactory subclass. The subclass can be used to create any
  number of different classes; the name of the required class is
  passed in the \a className argument, and the arguments to be passed
  to that class's constructor are passed in the \a arguments list. See
  QSArgument for further information on the arguments. \a context os
  the script QObject context in which the class has been instantiated,
  or 0 if the class has not been instantiated in a QObject context.

  Only QObject subclasses may be created in this way. This
  function returns an instance of the requested class.

  This function is called by the scripting engine, e.g. when it
  encounters code like this:
  \code
    var x = new ACppObject( arg1, arg2 ); // Qt Script
  \endcode

  The classes that a particular QSObjectFactory instance is capable of
  instantiating is returned by classes().

  If the argumenst are invalid or any other invalid operation happens,
  you can use throwError() to issue a Qt Script error.

*/

/*! \fn QStringList QSObjectFactory::classes() const

  This pure virtual function should be reimplemented in your
  QSObjectFactory subclass. It should return a list of the names of all
  the classes that your QSObjectFactory is capable of instantiating.

  This function is used by the scripting engine to find out which
  classes this QSObjectFactory can instantiate.

*/


/*! Constructor.
  To make a QSObjectFactory known to the scripting engine, instantiate
  a QSObjectFactory instance passing the interpreter \a i as an
  argument.  The scripting engine will then call the QSObjectFactory's
  classes() function to obtain the list of classes that this
  QSObjectFactory is capable of instantiating.
*/

QSObjectFactory::QSObjectFactory( QSInterpreter *i )
{
    ip = i ? i : QSInterpreter::defaultInterpreter();
    ip->interfaceFactory()->addConstructor( this );
}

/*!
    Informs the interpreter than an error occured. The error is
    treated like a normal Qt Script error. The error message is passed
    in \a message.
*/

void QSObjectFactory::throwError( const QString &message )
{
    ip->interpreter()->env()->throwError( message );
}
