/****************************************************************************
** $Id: qsproject.cpp  beta2   edited Jan 29 16:08 $
**
** Definition of the QSProject class.
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsproject.h"
#include "qsinterpreter.h"
#include "qseditor.h"
#include "../kernel/quickdebugger.h"
#include "../kernel/quickinterpreter.h"
#include "../kernel/quickobjects.h"
#include "../engine/qsfuncref.h"

#include <qapplication.h>
#include <qbuffer.h>
#include <qdir.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qmessagebox.h>

#include <stdlib.h>

// Designer files
#include <formfile.h>
#include <formwindow.h>
#include <mainwindow.h>
#include <project.h>
#include <resource.h>
#include <sourcefile.h>
#include <../interfaces/actioninterface.h>
#include <../interfaces/projectsettingsiface.h>
#include <../interfaces/designerinterface.h>

#ifdef QSA_EXPIRES
static void qsa_eval_check();
#endif

static QPluginManager<ActionInterface> *actionPluginManager = 0;
static QPluginManager<ProjectSettingsInterface> *projectSettingsPluginManager = 0;

// declared in QWidgetFactory
extern QString *qwf_plugin_dir;

static QSInterpreter *ide_interpreter = 0;

class QSADesignerInterface : public DesignerInterface
{
public:
    QSADesignerInterface( QSProject *proj ) : project( proj ) {}

    DesignerProject *currentProject() const { return project->project->iFace(); }
    DesignerFormWindow *currentForm() const { return 0; }
    DesignerSourceFile *currentSourceFile() const { return 0; }
    QPtrList<DesignerProject> projectList() const {
	QPtrList<DesignerProject> l;
	l.append( project->project->iFace() );
	return l;
    }
    void showStatusMessage( const QString &, int ) const {}
    DesignerDock *createDock() const { return 0; }
    DesignerOutputDock *outputDock() const { return 0; }
    void setModified( bool, QWidget * ) {}
    void updateFunctionList() {}

    void onProjectChange( QObject *, const char * ) {}
    void onFormChange( QObject *, const char * ) {}

    bool singleProjectMode() const { return TRUE; }

    QRESULT queryInterface( const QUuid &uuid, QUnknownInterface** iface ) {
	*iface = 0;

	if ( uuid == IID_QUnknown )
	    *iface = (QUnknownInterface*)this;
	else if ( uuid == IID_QComponentInformation )
	    *iface = (QComponentInformationInterface*)this;
	else if ( uuid == IID_Designer )
	    *iface = (DesignerInterface*)this;
	else
	    return QE_NOINTERFACE;

	(*iface)->addRef();
	return QS_OK;
    }
    Q_REFCOUNT

private:
    QSProject *project;

};

class QSProjectPrivate {
public:
};

/*!
  \class QSProject qsproject.h

  \brief The QSProject class provides control over a script project
  for the Qt Application Script interpreter.

  Qt Script for Applications (QSA) collects sets of functions and
  classes together in \e projects. Use open() to access a project,
  either from a file or from a byte array. If a file is used, QSA will
  automatically take care of saving user changes. The project's data
  can be retrieved into a byte array using projectData(). One project
  is current at any one time; close the current project with clear().

  The contents of the current project are available using forms() and
  classes() which return a list of all the forms and classes
  respectively.

  A new global function can be added with addFunction(); this function
  can also invoke \e{Qt Scripter} if required. \e{Qt Scripter} is
  invoked with openIDE() and closed with closeIDE().

  If \e{Qt Scripter} is too complex for your application (even with
  the GUI builder functionality and/or the debugger switched off), you
  can use QSEditor to create a source code editor for Qt Script (with
  syntax highlighting, completion, auto indentation, etc), which can
  be embedded into your application user interface.

  See the \link qsa.book Manual\endlink for more explanation and
  examples.

*/

/*! \fn bool QSProject::isOpened() const
  Returns whether this project is opened.
*/

/*! \fn QDateTime QSProject::lastModified() const
  Returns when this project was last modified.
*/

/*! \fn QString QSProject::projectName() const
  Returns the name of the project.
*/

/*! \fn QSInterpreter *QSProject::interpreter() const
  Returns the interpreter which is used to evaluate code
  of this project.
*/

/*! \fn QSEditor *QSProject::activeEditor() const
  Returns the editor which is currently editing the project,
  or 0 if no editor if active for that project.
*/

/*! \fn QPtrList<QSEditor> * QSProject::editors() const
  Returns all editors which work on this project.
*/

/*! \internal */

QSProject::QSProject()
    :
    designer( 0 ),
    project( 0 ),
    d( 0 ),
    interp( 0 ),
    listEditors( new QPtrList<QSEditor> ),
    running( FALSE ),
    opened( FALSE )
{
    setupPluginManagers();
}

/*! \internal
 Destructor */

QSProject::~QSProject()
{
    delete d;
    delete listEditors;
}

/*!  Returns all names of the forms of the project.

  \sa formSource(), formUi()
*/

QStringList QSProject::forms() const
{
    if ( !project )
	return QStringList();

    QPtrListIterator<FormFile> it = project->formFiles();
    QStringList lst;
    while ( it.current() ) {
	if ( it.current()->isFake() ) {
	    ++it;
	    continue;
	}
	lst << it.current()->formName();
	++it;
    }
    return lst;
}

/*! Returns a list for all files in this project */

QStringList QSProject::scriptFiles() const
{
    if ( !project )
	return QStringList();
    QPtrListIterator<SourceFile> it( project->sourceFiles() );
    QStringList lst;
    while ( it.current() ) {
	lst << it.current()->fileName();
	++it;
    }
    return lst;
}

/*! Returns the XML GUI description of the form called \a formName

  \sa forms(), formSource()
*/

QString QSProject::formUi( const QString &formName ) const
{
    if ( !project )
	return QString::null;
    QPtrListIterator<FormFile> it = project->formFiles();
    FormFile *ff = 0;
    while ( it.current() ) {
	if ( it.current()->isFake() ) {
	    ++it;
	    continue;
	}
	if ( QString( it.current()->formName() ) == formName ) {
	    ff = it.current();
	    break;
	}
	++it;
    }

    if ( !ff )
	return QString::null;

    if ( !ff->formWindow() ) {
	QFile f( ff->fileName() );
	if ( !f.open( IO_ReadOnly ) )
	    return QString::null;

	QTextStream ts( &f );
	return ts.read();
    }

    QCString s;
    QBuffer buffer( s );
    buffer.open( IO_WriteOnly );
    Resource resource( designer );
    resource.setWidget( ff->formWindow() );
    resource.save( &buffer );
    return QString( s );
}

/*! Returns the code for the form called \a formName

  \sa forms(), formUi()
*/

QString QSProject::formSource( const QString &formName ) const
{
    if ( !project )
	return QString::null;
    QPtrListIterator<FormFile> it = project->formFiles();
    FormFile *ff = 0;
    while ( it.current() ) {
	if ( it.current()->formName() == formName ) {
	    ff = it.current();
	    break;
	}
	++it;
    }

    if ( !ff )
	return QString::null;
    return ff->code();
}

/*! Returns the source of the file \a fileName in this project
 */

QString QSProject::source( const QString &fileName ) const
{
    if ( !project )
	return QString::null;
    SourceFile *sf = project->findSourceFile( fileName );
    if ( !sf )
	return QString::null;
    return sf->text();
}

/*!
  Returns the whole project as a data block (QByteArray).

  This is useful for keeping the project data with a document. For
  example, you could take the data returned by this function and store
  it in a document's file. Later, when the document was opened, you
  could extract this data from the document and pass it to the
  overload of open() that takes a QByteArray to reconstruct the
  project.

*/

QByteArray QSProject::projectData() const
{
    QByteArray ba;
    if ( !isOpened() )
	return ba;
    project->save();
    QFile f( project->fileName( TRUE ) );
    f.open( IO_ReadOnly );
    ba.resize( f.size() );
    f.readBlock( ba.data(), ba.size() );
    f.close();
    return ba;
}

/*! Adds the code \a code under the name \a fileName to the project as
  global code

  If there has been added code already under the name \a fileName, \a
  code is appended to this code if \a append is TRUE, otherwise the
  currently set code is replaced with \a code.

  \sa removeSource()
*/

void QSProject::addSource( const QString &code, const QString &fileName, bool append )
{
    if ( !project )
	return;
    int sourceId;
    interp->interpreter()->execute( 0, code, sourceId );
    SourceFile *sf = project->findSourceFile( project->makeRelative( fileName ) );
    if ( !sf )
	sf = new SourceFile( project->makeRelative( fileName ), FALSE, project );
    QString cd( code );
    if ( append )
	cd = sf->text() + "\n" + code;
    project->setModified( TRUE );
    sf->setText( cd );
    sf->setModified( TRUE );
    if ( !sf->isModified() || // this is a hack, look at SourceFile::isModified() for an explaination
	 !QFile::exists( project->makeAbsolute( fileName ) ) ) {
	QFile fl( project->makeAbsolute( fileName ) );
	if ( fl.open( IO_WriteOnly ) ) {
	    QTextStream ts( &fl );
	    ts << cd;
	}
    }
}

/*! Removes the code which was added under the filename \a fileName.

  \sa addSource()
*/

void QSProject::removeSource( const QString &fileName )
{
    if ( !project )
	return;
    SourceFile *sf = project->findSourceFile( project->makeRelative( fileName ) );
    if ( sf )
	project->removeSourceFile( sf );
}

/*! Loads the form \a fileName and adds it to the project.

  \sa removeForm()
*/

void QSProject::addForm( const QString &fileName )
{
    if ( !project )
	return;
    Resource resource( designer );
    FormFile *ff = new FormFile( project->makeRelative( fileName ),
				 FALSE, project );
    resource.load( ff, project );
}

/*! Removes the form which was added under the filename \a fileName.

  \sa addForm()
*/

void QSProject::removeForm( const QString &fileName )
{
    if ( !project )
	return;
    FormFile *ff = project->findFormFile( project->makeRelative( fileName ) );
    if ( ff )
	project->removeFormFile( ff );
}

/*!
  \overload

  Reads and parses the Qt Script for Applications script from the \a
  projectData QByteArray. \a projectName must be specified.

  If an error occurs while running or parsing the script, the error()
  signal is emitted, and further action may occur depending on the
  errorMode().

  After open() is called, functions, classes and forms defined in this
  project can be called and used inside call() and evaluate() calls.

  If the project has signal/slot connections (for example, a form that
  connects widget signals to Qt Script slots), then these connections
  are established as part of the open() process.

  The project always stops when the Qt Scripter is invoked, and
  continues when the Qt Scripter is closed.
*/

void QSProject::open( const QByteArray &projectData, const QString &projectName )
{
    QString qsa = QDir::homeDirPath() + QString( "/.qsa" );
    if ( !QFile::exists( qsa ) ) {
	QDir d;
	d.mkdir( qsa );
    }

    // ### avoid collisions
    QString fn = qsa + "/tmp_" + projectName + QString( ".qsa" );
    if ( QFile::exists( fn ) ) {
	QDir d;
	d.remove( fn );
    }
    QDir d;
    d.mkdir( qsa + "/tmp_tmp_" + projectName );
    if ( projectData.size() > 0 ) {
	QFile f( fn );
	if ( f.open( IO_WriteOnly ) ) {
	    f.writeBlock( projectData );
	    f.close();
	}
    }
    open( fn );
}

/*!
  Opens and parses the Qt Script for Applications script contained in
  the file, \a scriptProject. This overload of open() takes care of
  saving the project to the specified filename automatically. If an
  error occurs while running or parsing the script, actions according
  to the errorMode() are taken and the signal error() is emitted.

  After open() is called, functions, classes and forms defined in this
  project can be called and used using evaluate() and call().

  If the project has signal/slot connections (for example, a form that
  connects widget signals to Qt Script slots), then these connections
  are established as part of the open() process.

  The project always stops when Qt Scripter is opened, and continues
  when Qt Scripter is closed.
*/

void QSProject::open( const QString &scriptProject )
{
    interp->clear();

    QString qsa = QDir::homeDirPath() + QString( "/.qsa" );
    if ( !QFile::exists( qsa ) ) {
	QDir d;
	d.mkdir( qsa );
    }

    project = new Project( QFileInfo( scriptProject ).absFilePath(), "",
			   projectSettingsPluginManager, FALSE, "Qt Script" );

    if ( !QFile::exists( project->makeAbsolute( project->fileName() ) ) ) {
	SourceFile *f = new SourceFile( "main.qs", FALSE, project );
	f->setText( "// Put all global functions here\n" );
	f->setModified( TRUE );
	if ( designer )
	    designer->editSource( f );
    }

    if ( designer )
	designer->setSingleProject( project );

    // ### Fix this somehow...
    project->setObjects( interp->appObjs );

    connect( project, SIGNAL( runtimeError( const QString &, const QString &, int ) ),
	     interp, SLOT( runtimeError( const QString &, const QString &, int ) ) );
    connect( project, SIGNAL( runFinished() ),
	     interp, SLOT( runFinished() ) );
    connect( project, SIGNAL( newFormOpened( FormWindow * ) ),
	     this, SLOT( formAdded( FormWindow * ) ) );

    lastmod  = QFileInfo( scriptProject ).lastModified();
    projname = scriptProject;
    opened = TRUE;

    checkProject();
}

/*! \enum QSProject::IDEFlags

  The IDEFlags describe in which mode Qt Scripter is opened.

  \value GUIBuilder Opens Qt Scripter with the GUI Builder

  \value Debugger Opens Qt Scripter with the Debugger

  \value SignalHandlerTab Opens Qt Scripter with the possibility to
  connect C++ signals to Qt Script function via the user interface

  \value EditableEditor Opens Qt Scripter with editable editors

  \value FullIDE Convenience value to open Qt Scripter with
  all features

  \value OnlyEditor Convenience value to open Qt Scripter only with
  editor (including the signal handlers tab)

  \value OnlySimpleEditor Convenience value to open Qt Scripter only with
  a simple editor and nothing else

  \value OnlyEditorAndDebugger Convenience value to open Qt Scripter
  only with editor (including the signal handlers tab) and debugger
*/

/*!
  Opens Qt Scripter with the current scripting project (if there is
  one open), so that the project can be edited and debugged.

  \a flags is a value which can contains values of DeveloperFlags or'd
  together.

  If there is no project currently open this function does nothing.

  This function clears the interpreter, so code which has been passed
  into previous evaluate() calls will be lost.

  If you created QSEditors, the Qt Scripter is always opened with
  read only editors, even if EditableEditor is part of \a flags, so
  that the Qt Scripter can be used for debugging, but not editing.
*/

void QSProject::openIDE( int flags )
{
    if ( !isOpened() && isIDEOpen() )
	return;

    ide_interpreter = interp;

    setupDesigner();

    designer->showGUIStuff( ( flags & GUIBuilder )== GUIBuilder );
    designer->setEditorsReadOnly( ( flags & EditableEditor ) != EditableEditor && !!activeEditor() );
    designer->toggleSignalHandlers( ( flags & SignalHandlerTab ) == SignalHandlerTab );
    interp->interpreter()->emitToggleDebugger( ( flags & Debugger ) == Debugger );

    designer->show();
    designer->raise();
    designer->removeEventFilter( this );
    designer->installEventFilter( this );

    stopProject( TRUE );
}

/*!
  Closes Qt Scripter, if it is currently open.

  If there is no project currently open this function does nothing.

  This function clears the interpreter, so code which has been passed
  into previous evaluate() calls will be lost.
*/

void QSProject::closeIDE()
{
    if ( !isOpened() )
	return;

    if( !designer )
	return;

    designer->hide();

    ide_interpreter = 0;
    designer->setEditorsReadOnly( FALSE );
}

/*!
  Returns true if the IDE is currently open.
  \internal
*/
bool QSProject::isIDEOpen()
{
    return ide_interpreter;
}

/*!
  Returns the interpreterer currently used by the ide. When the
  ide is not open, this function returns 0
  \internal
*/
QSInterpreter* QSProject::ideInterpreter()
{
    if( !ide_interpreter )
	ide_interpreter = QSInterpreter::defaultInterpreter();
    return ide_interpreter;
}

QuickInterpreter *QSProject::ideQuickInterpreter()
{
    return ideInterpreter()->interpreter();
}




/*!
  A function called \a functionName is added to the current project.
  The name must match the regular expression
  \c{/^[A-Za-z_][A-Za-z_0-9]*$/}.

  The code for the function can be passed in the \a functionBody
  parameter. If \a open is TRUE (the default), then QSA Developer will
  be invoked so that the user can edit this function.  If \a
  functionBody was specified its text will appear in the editor; if
  the \a functionBody is empty a new empty function will be
  created. If the function \a functionName already exists in the
  project and \a open is TRUE, Qt Scripter will be invoked showing the
  already existing function in the editor (and any \a functionBody
  will be discarded).

  This function returns TRUE on success (i.e. the \a functionName was
  valid); otherwise returns FALSE.

  If there is no project currently open this function does nothing.
*/

bool QSProject::addFunction( const QString &functionName,
			     const QString &functionBody,
			     bool open )
{
    if ( !isOpened() )
	return FALSE;

    static QString letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUFWXYZ_";
    static QString numbers = "0123456789";
    for ( int i = 0; i < (int)functionName.length(); ++i ) {
	if ( i == 0 && letters.find( functionName[i] ) == -1 ||
	     letters.find( functionName[i] ) == -1 && numbers.find( functionName[i] ) == -1 ) {
#if defined(QT_CHECK_RANGE)
	    qWarning( "QSProject::addFunction: '%s' is not a valid function name",
		      functionName.latin1() );
#endif
	    return FALSE;
	}
    }

    if ( open )
	openIDE();
    project->addAndEditFunction( functionName, functionBody, open );
    return TRUE;
}

/*!
  Normally this function doesn't need to be called, since it is called
  automatically by the relevant functions (e.g. open(),
  call(), execute(), etc.). This function is useful in the case where
  an error has occurred and the script must be run again to
  reestablish the signal/slot connections between Qt and the script.
*/

void QSProject::reset()
{
    if ( !isOpened() )
	return;
    checkProject();
}

/*! Returns the code of the object \a object */

QString QSProject::objectSource( QObject *object ) const
{
    if( !project )
	return QString::null;
    FormFile *ff = project->fakeFormFileFor( object );
    if ( !ff )
	return QString::null;
    return ff->code();
}

/*! Adds the script code \a code in the context of \a object. \a
  object must have been available to the scripting language via
  addObject() before.

  If there has been set code already for that object \a object, \a
  code is appended to this code if \a append is TRUE, otherwise the
  currently set code is replaced with \a code.
*/

void QSProject::setObjectSource( QObject *object, const QString &code,
				     bool append )
{
    if ( !project )
	return;
    int sourceId;
    interp->interpreter()->execute( object, code, sourceId );
    interp->interpreter()->addSourceId( sourceId, object );
    FormFile *ff = project->fakeFormFileFor( object );
    if ( !ff )
	return;
    QString cd( code );
    if ( append && !ff->code().isEmpty() )
	cd = ff->code() + "\n" + cd;
    ff->parseCode( cd, TRUE );
    ff->setCode( cd );
    ff->setModified( TRUE );
    project->setModified( TRUE );
    if ( ff->editor() )
	ff->editor()->editorInterface()->setText( cd );
}

QSEditor* QSProject::activeEditor() const
{
    for ( QSEditor *e = listEditors->first(); e; e = listEditors->next() ) {
	if ( e->isActive() )
	    return e;
    }
    return 0;
}


/*!
  Sets \a inter to be the interpreter of this project.
*/
void QSProject::setInterpreter( QSInterpreter *inter )
{
    if( interp )
	qFatal( "QSProject::setInterpreter(), Project already has an interpreter" );
//     if( ide_interpreter )
// 	qWarning( "QSProject::setInterpreter(), "
// 		  "QSA does not support working with multiple interpreters" );
//     else
    ide_interpreter = inter;
    interp = inter;

}

QSInterpreter* QSProject::interpreter() const
{
    return interp;
}


/*! This function is called when a new QSEditor has been
  created.

  \internal
*/

void QSProject::registerEditor( QSEditor *editor )
{
    listEditors->append( editor );
}

/*! This function is called when a QSEditor has been
  detroyed.

  \internal
*/

void QSProject::unregisterEditor( QSEditor *editor )
{
    listEditors->removeRef( editor );
}

/*! \internal */
void QSProject::setupPluginManagers()
{
    if ( actionPluginManager )
	return;

#ifdef QSA_EXPIRES
    qsa_eval_check();
#endif

    if ( !qwf_plugin_dir )
	qwf_plugin_dir = new QString( "/qsa" );
    else
	*qwf_plugin_dir = QString( "/qsa" );

    MetaDataBase::setupInterfaceManagers( "/qsa" );
    projectSettingsPluginManager =
	new QPluginManager<ProjectSettingsInterface>( IID_ProjectSettings,
						      QApplication::libraryPaths(),
						      "/qsa" );

    // That part is hacky. To instantiate the debugger in QuickIDE, we
    // have to load the action plugins. This should be solved properly
    // by having a DebuggerInterface or so, as the way we do it is
    // just too hacky

    actionPluginManager = new QPluginManager<ActionInterface>( IID_Action,
						       QApplication::libraryPaths(),
						       "/qsa" );

    QStringList lst = actionPluginManager->featureList();
    QSADesignerInterface *dIface = new QSADesignerInterface( this );
    for ( QStringList::Iterator ait = lst.begin(); ait != lst.end(); ++ait ) {
	ActionInterface *iface = 0;
	actionPluginManager->queryInterface( *ait, &iface );
	if ( !iface )
	    continue;
	if ( !(*ait).startsWith( "Qt Script Debugger" ) )
	    continue;
	iface->connectTo( dIface );
	iface->release();
    }
}

/* !\internal */
bool QSProject::checkProject()
{
    QuickInterpreter *i = interp->interpreter();
    int iid = i->uniqueId();
    if ( opened && project ) {
	interp->hadError = FALSE;
	if ( !running ) {
	    interp->interpreter()->emitRunProject();
	    lastmod = QFileInfo( projname ).lastModified();
	} else if ( project->isModified() ) {
	    project->save();
	    interp->interpreter()->emitStopProject();
	    interp->interpreter()->emitRunProject();
	    lastmod = QFileInfo( projname ).lastModified();
	} else if ( QFileInfo( projname ).lastModified() != lastmod ) {
	    interp->interpreter()->emitStopProject();
	    interp->interpreter()->emitRunProject();
	    lastmod = QFileInfo( projname ).lastModified();
	} else if( interp->needRerun ) {
	    i->emitStopProject();
	    i->emitRunProject();
	}
	interp->needRerun = FALSE;
	running = !interp->hadError;
	interp->hadError = FALSE;
    }

    if ( iid != interp->interpreter()->uniqueId() ) {
	connect( interp->interpreter(), SIGNAL( runtimeError() ),
		 this, SLOT( runtimeError() ) );
	interp->readdEventHandlers();
    }

    return iid == interp->interpreter()->uniqueId();
}

/*! \internal */
void QSProject::setupDesigner()
{
    if ( designer )
	return;
    QApplication::setOverrideCursor( waitCursor );
    designer = new MainWindow( FALSE, TRUE, "/qsa" );
    designer->setCaption( tr( "Qt Scripter" ) );
    connect( designer, SIGNAL( destroyed( QObject * ) ),
	     this, SLOT( objectDestroyed( QObject * ) ) );
    QApplication::restoreOverrideCursor();

    designer->setSingleProject( project );
}

/*! \internal */
void QSProject::stopProject( bool force )
{
    if ( !isOpened() )
	return;
    if ( !force && ( !designer || !designer->isVisible() ) && !activeEditor() )
	return;
    // ### CONV
    interp->interpreter()->emitStopProject();
    interp->readdObjects();
    running = FALSE;
}


/* \internal */
void QSProject::closeProject()
{
    if( project )
	project->save();
    closeIDE();
    opened = FALSE;
    running = FALSE;
    if( project )
	project->removeTempProject();
}

/* \internal */
void QSProject::checkAndStopProject()
{
    interp->blockErrors = TRUE;
    checkProject();
    stopProject();
    interp->blockErrors = FALSE;
}

void QSProject::formAdded( FormWindow *fw )
{
    interp->interpreter()->addTopLevelObject( fw->mainContainer() );
}

void QSProject::objectDestroyed( QObject *o )
{
    if( o == designer )
	designer = 0;
}

/*! \reimp */

bool QSProject::eventFilter( QObject *o, QEvent *e )
{
    if ( o == designer && e->type() == QEvent::Hide ) {
	interp->blockErrors = TRUE;
	// the hide event has to be processed first
 	QTimer::singleShot( 0, this, SLOT( checkProject() ) );
	interp->blockErrors = FALSE;
    } else if ( o == designer && e->type() == QEvent::Show ) {
 	stopProject( TRUE );
    }

    return QObject::eventFilter( o, e );
}

#ifdef QSA_EXPIRES
void qsa_eval_check()
{
    QDate date( QSA_EXPIRES>>16, (QSA_EXPIRES&0xff00)>>8, (QSA_EXPIRES&0xff) );
    if ( date > QDate::currentDate() )
	return;
    QMessageBox::warning( 0,
			  QObject::tr( "End of Evaluation Period" ),
			  QObject::tr( "The evaluation period of QSA has expired.\n\n"
				       "Please check http://www.trolltech.com/products/qsa for updates\n"
				       "or contact sales@trolltech.com for further information" ),
			  QMessageBox::Ok,
			  QMessageBox::NoButton );
}
#endif
